/* $Id: XmlWidget.cpp,v 1.1 2003/10/06 23:31:32 palm Exp $ */
/* 
 * Copyright (C) 2003 Christer Palm <palm@nogui.se>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#include "XmlWidget.h"
#include <glade/glade-xml.h>

namespace XmlWidget
{

//
//	Base constructor
//
Base::Base(const std::string& filename, const std::string& widgetname)
	throw (Exception)
{
	try {
		xml = Gnome::Glade::Xml::create(filename);
	} catch (Gnome::Glade::XmlError& e) {
		throw Exception(e.what());
	}
	baseObject = glade_xml_get_widget(xml->gobj(), widgetname.c_str());
	if (!baseObject) {
		throw Exception("Base widget \"" + widgetname + "not found in glade file \"" +
			filename + "\".");
	}
}

//
//	Base::get_widget()
//
Gtk::Widget* Base::get_widget(const Glib::ustring& name)
	throw (Exception)
{
	Gtk::Widget* widget = xml->get_widget(name);
	if (!widget) {
		throw Exception("Widget \"" + name + "\" not found in glade file \"'" +
			xml->gobj()->filename + "\".");
	}
	return widget;
}

//
//	Base::get_widget_checked()
//
Gtk::Widget* Base::get_widget_checked(const Glib::ustring& name, GType type)
	throw (Exception)
{
	Gtk::Widget* widget = get_widget(name);
	if (!g_type_is_a(G_OBJECT_TYPE(widget->gobj()), type)) {
		throw Exception("Widget \"" + name + "\" (in glade file \"" +
			xml->gobj()->filename + "\") is of type " +
			G_OBJECT_TYPE_NAME(widget->gobj()) + " but " +
			g_type_name(type) + " was expected.");
	}
	return widget;
}

} // namespace XmlWidget
