/* $Id: XmlWidget.h,v 1.2 2003/10/06 23:34:54 palm Exp $ */
/* 
 * Copyright (C) 2003 Christer Palm <palm@nogui.se>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
#ifndef XMLWIDGET_H_INCLUDED
#define XMLWIDGET_H_INCLUDED

#include <gtkmm.h>
#include <libglademm/xml.h>

namespace XmlWidget
{

//
//	Exception class used by XmlWidget
//
class Exception : public Glib::Exception
{
	private:
		Glib::ustring msg;

	public:
		Exception(const Glib::ustring& msg_) throw()
			{ msg = msg_; }
		
		virtual ~Exception() throw()
			{ }

		Glib::ustring what() const
			{ return msg; }
};

//
//	Base class for XmlWidget
//
class Base
{
	private:
		Glib::RefPtr<Gnome::Glade::Xml> xml;
		GtkWidget* baseObject;

	protected:
		Base(const std::string& filename, const std::string& widgetname)
			throw (Exception);

		inline GtkWidget* getBaseObject()
			{ return baseObject; }

	private:
		Gtk::Widget* get_widget_checked(const Glib::ustring& name, GType type)
			throw (Exception);

	public:
		Gtk::Widget* get_widget(const Glib::ustring& name)
			throw (Exception);

		template <class T_Widget>
		inline T_Widget* get_widget(const Glib::ustring& name, T_Widget*& widget)
			throw (Exception)
		{
			widget = dynamic_cast<T_Widget*>
				(get_widget_checked(name, T_Widget::get_base_type()));
			if (!widget) {
				throw Exception("dynamic_cast<> failed.");
			}
			return widget;
		}
};

//
//	Convenience template for deriving new XmlWidget types
//
template<class T_widget>
class Adapter : public Base, public T_widget
{
	public:
		Adapter(const std::string& filename, const std::string& widgetname) :
			Base(filename, widgetname),
			T_widget((typename T_widget::BaseObjectType*)Base::getBaseObject())
			{ }
};

//
//	XmlWidget specializations
//
typedef Adapter<Gtk::Dialog> Dialog;
typedef Adapter<Gtk::Window> Window;

} // namespace XmlWidget

#endif // XMLWIDGET_H_INCLUDED
